// *********************************************************************
//
// BrianHG_DDR3_PLL.sv clock generator with DQS CLK phase
// control for write leveling, RQD 90 degree clock, and
// CMD_CLK system clock which can be configured to: 
//
//
// Version 1.1, July 22, 2021
//
// This version now output the individual DDR3_CLK, DDR3_CLK_WDQ, DDR3_CLK_RDQ,
// ***NEW : DDR3_CLK_50, DDR3_CLK_25, plus the original CMD_CLK which will now equal
//          DDR3_CLK, DDR3_CLK_50, or DDR3_CLK_25 depending on the interface speed.
//          _50 = Half frequency, _25 = Quarter frequency.
//
//
// Written by Brian Guralnick.
// For public use.
// Leave questions in the https://www.eevblog.com/forum/fpga/brianhg_ddr3_controller-open-source-ddr3-controller/
//
// *********************************************************************
//
//

module BrianHG_DDR3_PLL #(

parameter string     FPGA_VENDOR             = "Altera",       // Use ALTERA, INTEL, LATTICE or XILINX.
parameter            FPGA_FAMILY             = "MAX 10",       // (USE "SIM" for RTL simulation bypassing any HW dependent functions) With Altera, use Cyclone III, Cyclone IV, Cyclone V, MAX 10,....

// ****************  System clock generation and operation.

parameter int        CLK_KHZ_IN              = 50000,          // PLL source input clock frequency in KHz.
parameter int        CLK_IN_MULT             = 40,             // Multiply factor to generate the DDR MTPS speed divided by 2.
parameter int        CLK_IN_DIV              = 5,              // Divide factor to generate the DDR MTPS speed divided by 2.

parameter int        DDR_TRICK_MTPS_CAP      = 0,              // 0=off, Set a false PLL DDR data rate for the compiler to allow FPGA overclocking.  ***DO NOT USE.

parameter string     INTERFACE_SPEED         = "Full",         // Either "Full", "Half", or "Quarter" speed for the user interface clock.
                                                               // This will effect the controller's interface CMD_CLK output port frequency.
                                                               // "Full" means that CMD_CLK = DDR3_CLK, "Half" means CMD_CLK is half speed of DDR3_CLK...

parameter int        DDR3_WDQ_PHASE          = 90,             // 90, 0,45,90,...270 Select the write and write DQS output clock phase relative to the DDR3_CLK/CK#
parameter int        DDR3_RDQ_PHASE          = 0               // 45, 0,45,90,...270 Select the read latch clock for the read data and DQS input relative to the DDR3_CLK.
)
(
   RST_IN,                // Active high reset input.
   RST_OUT,               // Low after reset timer has been released
   CLK_IN,                // Input clock.

   DDR3_CLK,              // DDR3 CK clock running at 1/2 the DQ rate.
   DDR3_CLK_WDQ,          // DDR3 write data clock 90 degree out of phase running at 1/2 the DQ rate.
   DDR3_CLK_RDQ,          // DDR3 phase adjustable read data input clock running at 1/2 the DQ rate.
   DDR3_CLK_50,           // DDR3 clock running at 1/4 the DQ rate.
   DDR3_CLK_25,           // DDR3 clock running at 1/8 the DQ rate.

   CMD_CLK,               // Ram controller interface clock running at DDR3_CLK, or 1/2 DDR3_CLK, or 1/4 DDR3_CLK
   PLL_LOCKED,            // High when the PLL is functioning.
   
   phase_cs,              // Phase counter select, selects which C[#] to adjust.
   phase_step,            // Step a phase.
   phase_updn,            // Step direction.
   phase_sclk,            // phase interface clock.
   phase_done             // Goes low when ready.
);

input        RST_IN,CLK_IN;
(*preserve*) output logic RST_OUT;
output logic DDR3_CLK,DDR3_CLK_WDQ,DDR3_CLK_RDQ,DDR3_CLK_50,DDR3_CLK_25,CMD_CLK,PLL_LOCKED;
logic        RST_PLL;
input [2:0]  phase_cs;
input        phase_step,phase_updn,phase_sclk;
output       phase_done;

// Verify that the 'INTERFACE_SPEED' is equal to 'Half', 'Quarter', or 'Eighth'
generate
         if (INTERFACE_SPEED[0]!="F" && INTERFACE_SPEED[0]!="f" &&
             INTERFACE_SPEED[0]!="H" && INTERFACE_SPEED[0]!="h" &&
             INTERFACE_SPEED[0]!="Q" && INTERFACE_SPEED[0]!="q"     )  initial begin

// **********************************************
// ***  Unknown INTERFACE_SPEED *****************
// **********************************************
$warning("****************************************");
$warning("*** BrianHG_DDR3_PLL PARAMETER ERROR ***");
$warning("********************************************************************************");
$warning("*** BrianHG_DDR3_PLL parameter .INTERFACE_SPEED(\"%s\") is not supported. ***",INTERFACE_SPEED);
$warning("*** Only \"Full\", \"Half\", and \"Quarter\" speeds are supported.               ***");
$warning("********************************************************************************");
$error;
$stop;
// ************************************
// *** End Unknown INTERFACE_SPEED  ***
// ************************************
end
endgenerate


// **************************************************
// Make the CMD_CLK_DIV = 1x, 2x, or 4x CLK_IN_DIV
// **************************************************

localparam CMD_CLK_DIV = CLK_IN_DIV * (1 + (INTERFACE_SPEED[0]=="H" || INTERFACE_SPEED[0]=="h") + (3*(INTERFACE_SPEED[0]=="Q" || INTERFACE_SPEED[0]=="q")) ) ;

localparam int PLL1_OUT_TRUE_KHZ    = (CLK_KHZ_IN * CLK_IN_MULT / CLK_IN_DIV  ) ;    // Register the true DDR3_CLK frequency in KHz.
localparam int PLL1_OUT_CMD_CLK_KHZ = (CLK_KHZ_IN * CLK_IN_MULT / CMD_CLK_DIV ) ;    // Register the true CMD_CLK frequency in KHz.

// Generate a phony input clock frequency for the PLL to report to your FPGA compiler
// so it will believe the DDR3_DQ/DQS bus is working within the set rate.  This will
// cause improper timing reports and poor system stability.

localparam int PLL1_KHZ_IN_TRICK = CLK_KHZ_IN * DDR_TRICK_MTPS_CAP / PLL1_OUT_TRUE_KHZ * 500 ;
 
// Select the PLL1 source clock input frequency and input period in picoseconds based on DDR_TRICK_MTPS_CAP setting.
localparam int PLL1_in_KHz = (DDR_TRICK_MTPS_CAP==0 || DDR_TRICK_MTPS_CAP>(PLL1_OUT_TRUE_KHZ/500)) ? CLK_KHZ_IN : PLL1_KHZ_IN_TRICK ;
localparam int PLL1_inps   = (1.0E9 / PLL1_in_KHz) ; 



// ********************* Generate warning if the DDR_TRICK_MTPS_CAP is enabled. *********************
generate
if (DDR_TRICK_MTPS_CAP!=0 ) initial begin
if (DDR_TRICK_MTPS_CAP>=(PLL1_OUT_TRUE_KHZ/500)) begin
$warning("*****************************");
$warning("*** BrianHG_DDR3_PLL Info ***");
$warning("************************************************************************");
$warning("*** BrianHG_DDR3_PLL parameter .DDR_TRICK_MTPS_CAP setting of (%d) ***",12'(DDR_TRICK_MTPS_CAP));
$warning("*** was not used since the DDR MTPS frequency is only %d MTPS.     ***",12'(PLL1_OUT_TRUE_KHZ/500));
$warning("************************************************************************");
end else begin
$warning("********************************");
$warning("*** BrianHG_DDR3_PLL WARNING ***");
$warning("*******************************************************************************");
$warning("*** BrianHG_DDR3_PLL parameter .DDR_TRICK_MTPS_CAP setting of (%d)        ***",12'(DDR_TRICK_MTPS_CAP));
$warning("*** is in use since the true DDR3_DQ MTPS frequency is higher at %d MTPS. ***",12'(PLL1_OUT_TRUE_KHZ/500));
$warning("*** %s is being falsely instructed that the DDR3 clock is running      ***",FPGA_VENDOR);
$warning("*** slower to allow full compilation for DDR_DQ/DQS port speed limit.  The  ***");
$warning("*** timing report will only be accurate as long as the PLL source clock has ***");
$warning("*** the proper frequency set in the .sdc file.  Make sure this is EXACTLY   ***");
$warning("*** what you want to do.  Proper design functionality is not guaranteed.    ***");
$warning("*******************************************************************************");
end
end
endgenerate


localparam  DDR3_WDQ_PHASE_ps = (((1000000 / ( PLL1_in_KHz/1000 * CLK_IN_MULT / CLK_IN_DIV )) * DDR3_WDQ_PHASE / 360) -1) ;
localparam  DDR3_RDQ_PHASE_ps = (((1000000 / ( PLL1_in_KHz/1000 * CLK_IN_MULT / CLK_IN_DIV )) * DDR3_RDQ_PHASE / 360) -1) ;

localparam  int Altera_Dummy_String [0:399] = '{ // Seriously, this is the only way...  I feel like an idiot making this public...
"0000","0010","0020","0030","0040","0050","0060","0070","0080","0090","0100","0110","0120","0130","0140","0150","0160","0170","0180","0190",
"0200","0210","0220","0230","0240","0250","0260","0270","0280","0290","0300","0310","0320","0330","0340","0350","0360","0370","0380","0390",
"0400","0410","0420","0430","0440","0450","0460","0470","0480","0490","0500","0510","0520","0530","0540","0550","0560","0570","0580","0590",
"0600","0610","0620","0630","0640","0650","0660","0670","0680","0690","0700","0710","0720","0730","0740","0750","0760","0770","0780","0790",
"0800","0810","0820","0830","0840","0850","0860","0870","0880","0890","0900","0910","0920","0930","0940","0950","0960","0970","0980","0990",
"1000","1010","1020","1030","1040","1050","1060","1070","1080","1090","1100","1110","1120","1130","1140","1150","1160","1170","1180","1190",
"1200","1210","1220","1230","1240","1250","1260","1270","1280","1290","1300","1310","1320","1330","1340","1350","1360","1370","1380","1390",
"1400","1410","1420","1430","1440","1450","1460","1470","1480","1490","1500","1510","1520","1530","1540","1550","1560","1570","1580","1590",
"1600","1610","1620","1630","1640","1650","1660","1670","1680","1690","1700","1710","1720","1730","1740","1750","1760","1770","1780","1790",
"1800","1810","1820","1830","1840","1850","1860","1870","1880","1890","1900","1910","1920","1930","1940","1950","1960","1970","1980","1990",
"2000","2010","2020","2030","2040","2050","2060","2070","2080","2090","2100","2110","2120","2130","2140","2150","2160","2170","2180","2190",
"2200","2210","2220","2230","2240","2250","2260","2270","2280","2290","2300","2310","2320","2330","2340","2350","2360","2370","2380","2390",
"2400","2410","2420","2430","2440","2450","2460","2470","2480","2490","2500","2510","2520","2530","2540","2550","2560","2570","2580","2590",
"2600","2610","2620","2630","2640","2650","2660","2670","2680","2690","2700","2710","2720","2730","2740","2750","2760","2770","2780","2790",
"2800","2810","2820","2830","2840","2850","2860","2870","2880","2890","2900","2910","2920","2930","2940","2950","2960","2970","2980","2990",
"3000","3010","3020","3030","3040","3050","3060","3070","3080","3090","3100","3110","3120","3130","3140","3150","3160","3170","3180","3190",
"3200","3210","3220","3230","3240","3250","3260","3270","3280","3290","3300","3310","3320","3330","3340","3350","3360","3370","3380","3390",
"3400","3410","3420","3430","3440","3450","3460","3470","3480","3490","3500","3510","3520","3530","3540","3550","3560","3570","3580","3590",
"3600","3610","3620","3630","3640","3650","3660","3670","3680","3690","3700","3710","3720","3730","3740","3750","3760","3770","3780","3790",
"3800","3810","3820","3830","3840","3850","3860","3870","3880","3890","3900","3910","3920","3930","3940","3950","3960","3970","3980","3990" };

localparam  DDR3_WDQ_PHASE_pss = Altera_Dummy_String[DDR3_WDQ_PHASE_ps/10] ;
localparam  DDR3_RDQ_PHASE_pss = Altera_Dummy_String[DDR3_RDQ_PHASE_ps/10] ;

wire [4:0]  PLL1_clk_out;    // PLL has 5 outputs.

generate
// ******************************************************************************************************************************************
// ***  ALTERA/INTEL PLL ********************************************************************************************************************
// ******************************************************************************************************************************************
if (FPGA_VENDOR[0] == "A" || FPGA_VENDOR[0] == "a" || FPGA_VENDOR[0] == "I" || FPGA_VENDOR[0] == "i") begin 

// **********************************************************
// *** Begin Initiate Altera PLL for clk_pixel generation ***
// **********************************************************

 altpll HPLL1 ( .inclk ({1'b0, CLK_IN}),   .clk (PLL1_clk_out),
                .activeclock (),           .areset (RST_PLL),      .clkbad (),          .clkena ({6{1'b1}}),     .clkloss (),
                .clkswitch (1'b0),         .configupdate (1'b0),   .enable0 (),         .enable1 (),             .extclk (),
                .extclkena ({4{1'b1}}),    .fbin (1'b1),           .fbmimicbidir (),    .fbout (),               .fref (),
                .icdrclk (),               .locked (PLL_LOCKED),   .pfdena (1'b1),      .phasedone (phase_done), .phasestep (phase_step),
                .phaseupdown (phase_updn), .pllena (1'b1),         .scanaclr (1'b0),    .scanclk (phase_sclk),   .scanclkena (1'b1),
                .scandata (1'b0),          .scandataout (),        .scandone (),        .scanread (1'b0),        .scanwrite (1'b0),
                .sclkout0 (),              .sclkout1 (),           .vcooverrange (),    .vcounderrange (),       .phasecounterselect (phase_cs));
 defparam
  HPLL1.bandwidth_type = "AUTO",          HPLL1.inclk0_input_frequency = PLL1_inps, HPLL1.compensate_clock = "CLK0",        HPLL1.lpm_hint = "CBX_MODULE_PREFIX=HDMI_PLL",
  HPLL1.clk0_divide_by = CLK_IN_DIV,      HPLL1.clk0_duty_cycle = 50,               HPLL1.clk0_multiply_by = CLK_IN_MULT,   HPLL1.clk0_phase_shift = "0",
  HPLL1.clk1_divide_by = CLK_IN_DIV,      HPLL1.clk1_duty_cycle = 50,               HPLL1.clk1_multiply_by = CLK_IN_MULT,   HPLL1.clk1_phase_shift = DDR3_WDQ_PHASE_pss,
  HPLL1.clk2_divide_by = CLK_IN_DIV,      HPLL1.clk2_duty_cycle = 50,               HPLL1.clk2_multiply_by = CLK_IN_MULT,   HPLL1.clk2_phase_shift = DDR3_RDQ_PHASE_pss,
  HPLL1.clk3_divide_by = CLK_IN_DIV*2,    HPLL1.clk3_duty_cycle = 50,               HPLL1.clk3_multiply_by = CLK_IN_MULT,   HPLL1.clk3_phase_shift = "0",
  HPLL1.clk4_divide_by = CLK_IN_DIV*4,    HPLL1.clk4_duty_cycle = 50,               HPLL1.clk4_multiply_by = CLK_IN_MULT,   HPLL1.clk4_phase_shift = "0",

  HPLL1.lpm_type         = "altpll",      HPLL1.operation_mode    = "NORMAL",       HPLL1.pll_type         = "AUTO",        HPLL1.port_activeclock        = "PORT_UNUSED",
  HPLL1.port_areset      = "PORT_USED",   HPLL1.port_clkbad0      = "PORT_UNUSED",  HPLL1.port_clkbad1     = "PORT_UNUSED", HPLL1.port_clkloss            = "PORT_UNUSED",
  HPLL1.port_clkswitch   = "PORT_UNUSED", HPLL1.port_configupdate = "PORT_UNUSED",  HPLL1.port_fbin        = "PORT_UNUSED", HPLL1.port_inclk0             = "PORT_USED",
  HPLL1.port_inclk1      = "PORT_UNUSED", HPLL1.port_locked       = "PORT_USED",    HPLL1.port_pfdena      = "PORT_UNUSED", HPLL1.port_phasecounterselect = "PORT_USED",
  HPLL1.port_phasedone   = "PORT_USED",   HPLL1.port_phasestep    = "PORT_USED",    HPLL1.port_phaseupdown = "PORT_USED",   HPLL1.port_pllena             = "PORT_UNUSED",
  HPLL1.port_scanaclr    = "PORT_UNUSED", HPLL1.port_scanclk      = "PORT_USED",    HPLL1.port_scanclkena  = "PORT_UNUSED", HPLL1.port_scandata           = "PORT_UNUSED",
  HPLL1.port_scandataout = "PORT_UNUSED", HPLL1.port_scandone     = "PORT_UNUSED",  HPLL1.port_scanread    = "PORT_UNUSED", HPLL1.port_scanwrite          = "PORT_UNUSED",
  HPLL1.port_clk0        = "PORT_USED",   HPLL1.port_clk1         = "PORT_USED",    HPLL1.port_clk2        = "PORT_USED",   HPLL1.port_clk3               = "PORT_USED",
  HPLL1.port_clk4        = "PORT_USED",   HPLL1.port_clk5         = "PORT_UNUSED",  HPLL1.port_clkena0     = "PORT_UNUSED", HPLL1.port_clkena1            = "PORT_UNUSED",
  HPLL1.port_clkena2     = "PORT_UNUSED", HPLL1.port_clkena3      = "PORT_UNUSED",  HPLL1.port_clkena4     = "PORT_UNUSED", HPLL1.port_clkena5            = "PORT_UNUSED",
  HPLL1.port_extclk0     = "PORT_UNUSED", HPLL1.port_extclk1      = "PORT_UNUSED",  HPLL1.port_extclk2     = "PORT_UNUSED", HPLL1.port_extclk3            = "PORT_UNUSED",
  HPLL1.width_clock      = 5,             HPLL1.width_phasecounterselect = 3,       HPLL1.self_reset_on_loss_lock = "OFF",  HPLL1.intended_device_family = FPGA_FAMILY;
// *******************************
// *** End Initiate Altera PLL ***
// *******************************

// ******************************************************************************************************************************************
// ***  LATTICE PLL ********************************************************************************************************************
// ******************************************************************************************************************************************
// end else if (FPGA_VENDOR[0] == "L" || FPGA_VENDOR[0] == "l") begin 


// ********************************
// *** End Initiate Lattice PLL ***
// ********************************
// ******************************************************************************************************************************************
// ***  Xilinx PLL ********************************************************************************************************************
// ******************************************************************************************************************************************
// end else if (FPGA_VENDOR[0] == "X" || FPGA_VENDOR[0] == "x") begin 


// ********************************
// *** End Initiate Xilinx PLL ***
// ********************************
end else initial begin
// ******************************************************************************************************************************************
// ***  Unknown FPGA Vendor **************************************************************************************************************
// ******************************************************************************************************************************************
$warning("****************************************");
$warning("*** BrianHG_DDR3_PLL PARAMETER ERROR ***");
$warning("****************************************************************************");
$warning("*** BrianHG_DDR3_PLL parameter .FPGA_VENDOR(\"%s\") is not supported. ***",FPGA_VENDOR);
$warning("*** Only supported vendors \"Altera\" or \"Intel\".                          ***");
$warning("****************************************************************************");
$error;
$stop;
// ********************************
// *** End Unknown FPGA Vendor  ***
// ********************************
end
endgenerate

assign   DDR3_CLK      = PLL1_clk_out[0];      // DDR3 CK clock running at 1/2 the DQ rate.
assign   DDR3_CLK_WDQ = PLL1_clk_out[1];      // DDR3 write data clock 90 degree out of phase running at 1/2 the DQ rate.
assign   DDR3_CLK_RDQ = PLL1_clk_out[2];      // DDR3 phase adjustable read data input clock running at 1/2 the DQ rate.
assign   DDR3_CLK_50   = PLL1_clk_out[3];      // DDR3 clock running at 1/4 the DQ rate.
assign   DDR3_CLK_25   = PLL1_clk_out[4];      // DDR3 clock running at 1/8 the DQ rate.


localparam CMD_CLK_SEL = ((INTERFACE_SPEED[0]=="H" || INTERFACE_SPEED[0]=="h") * 3) + ((INTERFACE_SPEED[0]=="Q" || INTERFACE_SPEED[0]=="q")*4) ;
assign     CMD_CLK     = PLL1_clk_out[CMD_CLK_SEL] ;

generate
initial begin
$warning("*****************************");
$warning("*** BrianHG_DDR3_PLL Info ***");
$warning("*********************************************");
$warning("***      CLK_IN           = %d MHz.     ***",12'(CLK_KHZ_IN/1000));
$warning("***      DDR3_RDQ/WDQ     = %d MTPS.    ***",12'(PLL1_OUT_TRUE_KHZ/500));
$warning("***      DDR3_CLK/RDQ/WDQ = %d MHz.     ***",12'(PLL1_OUT_TRUE_KHZ/1000));
$warning("***      DDR3_WDQ_PHASE   = %d degrees.  ***",9'(DDR3_WDQ_PHASE));
$warning("*** True DDR3_WDQ_PHASE   =  %s ps.     ***",DDR3_WDQ_PHASE_pss);
$warning("***      DDR3_RDQ_PHASE   = %d degrees.  ***",9'(DDR3_RDQ_PHASE));
$warning("*** True DDR3_RDQ_PHASE   =  %s ps.     ***",DDR3_RDQ_PHASE_pss);
$warning("***      CMD_CLK          = %d MHz.     ***",12'(PLL1_OUT_CMD_CLK_KHZ/1000));
$warning("***      DDR3_CLK_50      = %d MHz.     ***",12'(PLL1_OUT_TRUE_KHZ/2000));
$warning("***      DDR3_CLK_25      = %d MHz.     ***",12'(PLL1_OUT_TRUE_KHZ/4000));
$warning("*********************************************");
end
endgenerate


// **************************************************************************
// Generate an extended reset pulse for at least an extra 64+64 CLK_IN cycles.
// **************************************************************************
localparam         RST_BITS     = 4;
logic [RST_BITS:0] rst_counter  = 0;
assign             RST_PLL      = RST_IN || !rst_counter[RST_BITS];
logic [RST_BITS:0] rst_counter2 = 0;

always @(posedge CLK_IN) begin
     if (RST_IN)                  rst_counter  <= 0;
else if (!rst_counter[RST_BITS])  rst_counter  <= rst_counter + 1'b1;

     if (RST_IN || !PLL_LOCKED )  rst_counter2 <= 0;
else if (!rst_counter2[RST_BITS]) rst_counter2 <= rst_counter2 + 1'b1;

RST_OUT <= !rst_counter2[RST_BITS];
end

endmodule
